#include "aron_conversions.h"

#include <RobotAPI/libraries/aron/common/aron_conversions.h>
#include <RobotAPI/libraries/armem_objects/aron_conversions.h>
#include <RobotAPI/libraries/core/Pose.h>
#include <RobotAPI/libraries/ArmarXObjects/aron_conversions.h>
#include <ArmarXCore/core/exceptions/local/UnexpectedEnumValueException.h>



void armarx::grasping::fromAron(const armarx::grasping::arondto::BoundingBox& dto,
                                armarx::grasping::BoundingBox& bo)
{
    bo = BoundingBox(toIce(dto.center), toIce(dto.ha1), toIce(dto.ha2), toIce(dto.ha3));
}

void armarx::grasping::toAron(armarx::grasping::arondto::BoundingBox& dto,
                              const armarx::grasping::BoundingBox& bo)
{
    dto.center = fromIce(bo.center);
    dto.ha1 = fromIce(bo.ha1);
    dto.ha2 = fromIce(bo.ha2);
    dto.ha3 = fromIce(bo.ha3);
}

void armarx::grasping::fromAron(const armarx::grasping::arondto::GraspCandidateSourceInfo& dto,
                                armarx::grasping::GraspCandidateSourceInfo& bo)
{
    bo.bbox = new BoundingBox();
    fromAron(dto.bbox, *bo.bbox);
    bo.referenceObjectName = dto.referenceObjectName;
    bo.referenceObjectPose = toIce(dto.referenceObjectPose);
    bo.segmentationLabelID = dto.segmentationLabelID;
}

void armarx::grasping::toAron(armarx::grasping::arondto::GraspCandidateSourceInfo& dto,
                              const armarx::grasping::GraspCandidateSourceInfo& bo)
{
    toAron(dto.bbox, *bo.bbox);
    dto.referenceObjectName = bo.referenceObjectName;
    dto.referenceObjectPose = fromIce(bo.referenceObjectPose);
    dto.segmentationLabelID = bo.segmentationLabelID;

}

void armarx::grasping::fromAron(const armarx::grasping::arondto::GraspCandidateReachabilityInfo& dto,
                                armarx::grasping::GraspCandidateReachabilityInfo& bo)
{
    bo = GraspCandidateReachabilityInfo(dto.reachable, dto.minimumJointLimitMargin, dto.jointLimitMargins,
                                        dto.maxPosError, dto.maxOriError);
}

void armarx::grasping::toAron(armarx::grasping::arondto::GraspCandidateReachabilityInfo& dto,
                              const armarx::grasping::GraspCandidateReachabilityInfo& bo)
{
    dto.jointLimitMargins = bo.jointLimitMargins;
    dto.maxOriError = bo.maxOriError;
    dto.maxPosError = bo.maxPosError;
    dto.minimumJointLimitMargin = bo.minimumJointLimitMargin;
    dto.reachable = bo.reachable;
}

void armarx::grasping::fromAron(const armarx::grasping::arondto::GraspCandidate& dto,
                                armarx::grasping::GraspCandidate& bo)
{
    bo = GraspCandidate();
    bo.graspPose = toIce(dto.graspPose);
    bo.robotPose = toIce(dto.robotPose);
    bo.tcpPoseInHandRoot = toIce(dto.tcpPoseInHandRoot);
    bo.approachVector = toIce(dto.approachVector);
    bo.sourceFrame = dto.sourceFrame;
    bo.targetFrame = dto.targetFrame;
    bo.side = dto.side;
    bo.graspSuccessProbability = dto.graspSuccessProbability;
    fromAron(dto.objectType, bo.objectType);
    if (dto.executionHintsValid)
    {
        bo.executionHints = new GraspCandidateExecutionHints();
        fromAron(dto.executionHints, *bo.executionHints);
    }
    else
    {
        bo.executionHints = nullptr;
    }
    bo.groupNr = dto.groupNr;
    bo.providerName = dto.providerName;
    if (dto.reachabilityInfoValid)
    {
        bo.reachabilityInfo = new GraspCandidateReachabilityInfo();
        fromAron(dto.reachabilityInfo, *bo.reachabilityInfo);
    }
    else
    {
        bo.reachabilityInfo = nullptr;
    }
    if (dto.sourceInfoValid)
    {
        bo.sourceInfo = new GraspCandidateSourceInfo();
        fromAron(dto.sourceInfo, *bo.sourceInfo);
    }
    else
    {
        bo.sourceInfo = nullptr;
    }
}

void armarx::grasping::toAron(armarx::grasping::arondto::GraspCandidate& dto,
                              const armarx::grasping::GraspCandidate& bo)
{
    dto.approachVector = fromIce(bo.approachVector);
    if (bo.executionHints)
    {
        dto.executionHintsValid = true;
        toAron(dto.executionHints, *bo.executionHints);
    }
    else
    {
        dto.executionHintsValid = false;
        dto.executionHints.toAron();
    }
    dto.graspPose = fromIce(bo.graspPose);
    dto.graspSuccessProbability = bo.graspSuccessProbability;
    dto.groupNr = bo.groupNr;
    toAron(dto.objectType, bo.objectType);
    dto.providerName = bo.providerName;
    if (bo.reachabilityInfo)
    {
        dto.reachabilityInfoValid = true;
        toAron(dto.reachabilityInfo, *bo.reachabilityInfo);
    }
    else
    {
        dto.reachabilityInfoValid = false;
        dto.reachabilityInfo.toAron();
    }
    dto.robotPose = fromIce(bo.robotPose);
    dto.tcpPoseInHandRoot = fromIce(bo.tcpPoseInHandRoot);
    dto.side = bo.side;
    dto.sourceFrame = bo.sourceFrame;
    if (bo.sourceInfo)
    {
        dto.sourceInfoValid = true;
        toAron(dto.sourceInfo, *bo.sourceInfo);
    }
    else
    {
        dto.sourceInfoValid = false;
        dto.sourceInfo.toAron();
    }
    dto.targetFrame = bo.targetFrame;
}

void armarx::grasping::fromAron(const armarx::grasping::arondto::BimanualGraspCandidate& dto,
                                armarx::grasping::BimanualGraspCandidate& bo)
{
    bo = BimanualGraspCandidate();
    bo.graspPoseRight = toIce(dto.graspPoseRight);
    bo.graspPoseLeft = toIce(dto.graspPoseLeft);
    bo.robotPose = toIce(dto.robotPose);
    bo.tcpPoseInHandRootRight = toIce(dto.tcpPoseInHandRootRight);
    bo.tcpPoseInHandRootLeft = toIce(dto.tcpPoseInHandRootLeft);
    bo.approachVectorRight = toIce(dto.approachVectorRight);
    bo.approachVectorLeft = toIce(dto.approachVectorLeft);
    bo.inwardsVectorRight = toIce(dto.inwardsVectorRight);
    bo.inwardsVectorLeft = toIce(dto.inwardsVectorLeft);
    bo.sourceFrame = dto.sourceFrame;
    bo.targetFrame = dto.targetFrame;
    fromAron(dto.objectType, bo.objectType);
    if (dto.executionHintsRightValid)
    {
        bo.executionHintsRight = new GraspCandidateExecutionHints();
        fromAron(dto.executionHintsRight, *bo.executionHintsRight);
    }
    else
    {
        bo.executionHintsRight = nullptr;
    }
    if (dto.executionHintsLeftValid)
    {
        bo.executionHintsLeft = new GraspCandidateExecutionHints();
        fromAron(dto.executionHintsLeft, *bo.executionHintsLeft);
    }
    else
    {
        bo.executionHintsLeft = nullptr;
    }
    bo.groupNr = dto.groupNr;
    bo.providerName = dto.providerName;
    if (dto.reachabilityInfoRightValid)
    {
        bo.reachabilityInfoRight = new GraspCandidateReachabilityInfo();
        fromAron(dto.reachabilityInfoRight, *bo.reachabilityInfoRight);
    }
    else
    {
        bo.reachabilityInfoRight = nullptr;
    }
    if (dto.reachabilityInfoLeftValid)
    {
        bo.reachabilityInfoLeft = new GraspCandidateReachabilityInfo();
        fromAron(dto.reachabilityInfoLeft, *bo.reachabilityInfoLeft);
    }
    else
    {
        bo.reachabilityInfoLeft = nullptr;
    }
    if (dto.sourceInfoValid)
    {
        bo.sourceInfo = new GraspCandidateSourceInfo();
        fromAron(dto.sourceInfo, *bo.sourceInfo);
    }
    else
    {
        bo.sourceInfo = nullptr;
    }
    bo.graspName = dto.graspName;
}

void armarx::grasping::toAron(armarx::grasping::arondto::BimanualGraspCandidate& dto,
                              const armarx::grasping::BimanualGraspCandidate& bo)
{
    dto.approachVectorRight = fromIce(bo.approachVectorRight);
    dto.approachVectorLeft = fromIce(bo.approachVectorLeft);
    if (bo.executionHintsRight)
    {
        dto.executionHintsRightValid = true;
        toAron(dto.executionHintsRight, *bo.executionHintsRight);
    }
    else
    {
        dto.executionHintsRightValid = false;
        dto.executionHintsRight.toAron();
    }
    if (bo.executionHintsLeft)
    {
        dto.executionHintsLeftValid = true;
        toAron(dto.executionHintsLeft, *bo.executionHintsLeft);
    }
    else
    {
        dto.executionHintsLeftValid = false;
        dto.executionHintsLeft.toAron();

    }
    dto.graspPoseRight = fromIce(bo.graspPoseRight);
    dto.graspPoseLeft = fromIce(bo.graspPoseLeft);
    dto.groupNr = bo.groupNr;
    toAron(dto.objectType, bo.objectType);
    dto.providerName = bo.providerName;
    if (bo.reachabilityInfoRight)
    {
        dto.reachabilityInfoRightValid = true;
        toAron(dto.reachabilityInfoRight, *bo.reachabilityInfoRight);
    }
    else
    {
        dto.reachabilityInfoRightValid = false;
        dto.reachabilityInfoRight.toAron();
    }
    if (bo.reachabilityInfoLeft)
    {
        dto.reachabilityInfoLeftValid = true;
        toAron(dto.reachabilityInfoLeft, *bo.reachabilityInfoLeft);
    }
    else
    {
        dto.reachabilityInfoLeftValid = false;
        dto.reachabilityInfoLeft.toAron();
    }
    dto.robotPose = fromIce(bo.robotPose);
    dto.sourceFrame = bo.sourceFrame;
    if (bo.sourceInfo)
    {
        dto.sourceInfoValid = true;
        toAron(dto.sourceInfo, *bo.sourceInfo);
    }
    else
    {
        dto.sourceInfoValid = false;
        dto.sourceInfo.toAron();
    }
    dto.targetFrame = bo.targetFrame;
    dto.inwardsVectorRight = fromIce(bo.inwardsVectorRight);
    dto.inwardsVectorLeft = fromIce(bo.inwardsVectorLeft);
    dto.graspName = bo.graspName;
}

void armarx::grasping::fromAron(const armarx::grasping::arondto::GraspCandidateExecutionHints& dto,
                                armarx::grasping::GraspCandidateExecutionHints& bo)
{
    bo = GraspCandidateExecutionHints();
    fromAron(dto.approach, bo.approach);
    fromAron(dto.preshape, bo.preshape);
    bo.graspTrajectoryName = dto.graspTrajectoryName;
}

void armarx::grasping::toAron(armarx::grasping::arondto::GraspCandidateExecutionHints& dto,
                              const armarx::grasping::GraspCandidateExecutionHints& bo)
{
    toAron(dto.approach, bo.approach);
    toAron(dto.preshape, bo.preshape);
    dto.graspTrajectoryName = bo.graspTrajectoryName;
}

void armarx::grasping::fromAron(const armarx::grasping::arondto::ApproachType& dto,
                                armarx::grasping::ApproachType& bo)
{
    switch (dto.value)
    {
        case arondto::ApproachType::AnyApproach:
            bo = ApproachType::AnyApproach;
            return;
        case arondto::ApproachType::TopApproach:
            bo = ApproachType::TopApproach;
            return;
        case arondto::ApproachType::SideApproach:
            bo = ApproachType::SideApproach;
            return;
    }
    ARMARX_UNEXPECTED_ENUM_VALUE(arondto::ObjectType, dto.value);

}

void armarx::grasping::toAron(armarx::grasping::arondto::ApproachType& dto,
                              const armarx::grasping::ApproachType& bo)
{
    switch (bo)
    {
        case ApproachType::AnyApproach:
            dto.value = arondto::ApproachType::AnyApproach;
            return;
        case ApproachType::TopApproach:
            dto.value = arondto::ApproachType::TopApproach;
            return;
        case ApproachType::SideApproach:
            dto.value = arondto::ApproachType::SideApproach;
            return;
    }
    ARMARX_UNEXPECTED_ENUM_VALUE(ObjectTypeEnum, bo);

}

void armarx::grasping::fromAron(const armarx::grasping::arondto::ApertureType& dto,
                                armarx::grasping::ApertureType& bo)
{
    switch (dto.value)
    {
        case arondto::ApertureType::AnyAperture:
            bo = ApertureType::AnyAperture;
            return;
        case arondto::ApertureType::OpenAperture:
            bo = ApertureType::OpenAperture;
            return;
        case arondto::ApertureType::PreshapedAperture:
            bo = ApertureType::PreshapedAperture;
            return;
    }
    ARMARX_UNEXPECTED_ENUM_VALUE(arondto::ObjectType, dto.value);
}

void armarx::grasping::toAron(armarx::grasping::arondto::ApertureType& dto,
                              const armarx::grasping::ApertureType& bo)
{
    switch (bo)
    {
        case ApertureType::AnyAperture:
            dto.value = arondto::ApertureType::AnyAperture;
            return;
        case ApertureType::OpenAperture:
            dto.value = arondto::ApertureType::OpenAperture;
            return;
        case ApertureType::PreshapedAperture:
            dto.value = arondto::ApertureType::PreshapedAperture;
            return;
    }
    ARMARX_UNEXPECTED_ENUM_VALUE(ObjectTypeEnum, bo);
}
