#include "ice_conversions.h"

#include <RobotAPI/libraries/aron/core/data/variant/container/Dict.h>


void IceUtil::toIce(long& ice, const IceUtil::Time& time)
{
    ice = time.toMicroSeconds();
}

void IceUtil::fromIce(const long& ice, IceUtil::Time& time)
{
    time = Time::microSeconds(ice);
}

namespace armarx
{

    void armem::toIce(data::MemoryID& ice, const MemoryID& id)
    {
        ice.memoryName = id.memoryName;
        ice.coreSegmentName = id.coreSegmentName;
        ice.providerSegmentName = id.providerSegmentName;
        ice.entityName = id.entityName;
        toIce(ice.timestampMicroSeconds, id.timestamp);
        ice.instanceIndex = id.instanceIndex;
    }

    void armem::fromIce(const data::MemoryID& ice, MemoryID& id)
    {
        id.memoryName = ice.memoryName;
        id.coreSegmentName = ice.coreSegmentName;
        id.providerSegmentName = ice.providerSegmentName;
        id.entityName = ice.entityName;
        fromIce(ice.timestampMicroSeconds, id.timestamp);
        id.instanceIndex = ice.instanceIndex;
    }

    void armem::fromIce(const data::Commit& ice, Commit& commit)
    {
        commit.updates.clear();
        for (const auto& ice_update : ice.updates)
        {
            EntityUpdate& update = commit.updates.emplace_back();
            fromIce(ice_update, update);
        }
    }

    void armem::toIce(data::Commit& ice, const Commit& commit)
    {
        ice.updates.clear();
        for (const auto& update : commit.updates)
        {
            data::EntityUpdate& ice_update = ice.updates.emplace_back();
            toIce(ice_update, update);
        }
    }

    void armem::fromIce(const data::CommitResult& ice, CommitResult& result)
    {
        result.results.clear();
        for (const auto& ice_res : ice.results)
        {
            EntityUpdateResult& res = result.results.emplace_back();
            fromIce(ice_res, res);
        }
    }

    void armem::toIce(data::CommitResult& ice, const CommitResult& result)
    {
        ice.results.clear();
        for (const auto& res : result.results)
        {
            data::EntityUpdateResult& ice_res = ice.results.emplace_back();
            toIce(ice_res, res);
        }
    }

    void armem::fromIce(const data::EntityUpdate& ice, EntityUpdate& update)
    {
        fromIce(ice.entityID, update.entityID);

        update.instancesData.clear();
        update.instancesData.reserve(ice.instancesData.size());
        std::transform(ice.instancesData.begin(), ice.instancesData.end(), std::back_inserter(update.instancesData),
                       aron::data::Dict::FromAronDictPtr);

        update.timeCreated = Time::microSeconds(ice.timeCreatedMicroSeconds);

        update.confidence = ice.confidence;
        update.timeSent = Time::microSeconds(ice.timeSentMicroSeconds);
    }

    void armem::toIce(data::EntityUpdate& ice, const EntityUpdate& update)
    {
        toIce(ice.entityID, update.entityID);

        ice.instancesData.clear();
        ice.instancesData.reserve(update.instancesData.size());
        std::transform(update.instancesData.begin(), update.instancesData.end(), std::back_inserter(ice.instancesData),
                       aron::data::Dict::ToAronDictPtr);

        ice.timeCreatedMicroSeconds = update.timeCreated.toMicroSeconds();

        ice.confidence = update.confidence;
        ice.timeSentMicroSeconds = update.timeSent.toMicroSeconds();
    }

    void armem::fromIce(const data::EntityUpdateResult& ice, EntityUpdateResult& result)
    {
        result.success = ice.success;
        fromIce(ice.snapshotID, result.snapshotID);
        result.timeArrived = Time::microSeconds(ice.timeArrivedMicroSeconds);
        result.errorMessage = ice.errorMessage;
    }

    void armem::toIce(data::EntityUpdateResult& ice, const EntityUpdateResult& result)
    {
        ice.success = result.success;
        toIce(ice.snapshotID, result.snapshotID);
        ice.timeArrivedMicroSeconds = result.timeArrived.toMicroSeconds();
        ice.errorMessage = result.errorMessage;
    }

    void armem::fromIce(const data::Commit& ice, Commit& commit, Time timeArrived)
    {
        commit.updates.clear();
        for (const auto& ice_update : ice.updates)
        {
            EntityUpdate& update = commit.updates.emplace_back(EntityUpdate());
            fromIce(ice_update, update, timeArrived);
        }
    }

    void armem::fromIce(const data::EntityUpdate& ice, EntityUpdate& update, Time timeArrived)
    {
        armem::fromIce(ice, update);
        update.timeArrived = timeArrived;
    }

}

