#pragma once

#include "MemoryBase.h"

namespace armarx::armem::server::ltm
{
    template <class _CoreSegmentT>
    class BufferedMemoryBase : virtual public MemoryBase<_CoreSegmentT>
    {
    public:
        using MemoryBase<_CoreSegmentT>::MemoryBase;
        BufferedMemoryBase(const MemoryID&) = delete; // remove non-default constructor

        armem::wm::Memory getBuffer() const
        {
            std::lock_guard l(this->ltm_mutex);
            return buffer;
        }

        void directlyStore(const armem::wm::Memory& memory)
        {
            TIMING_START(LTM_Memory_DirectlyStore);
            _directlyStore(memory);
            TIMING_END(LTM_Memory_DirectlyStore);
        }

        void storeBuffer()
        {
            this->directlyStore(buffer);
            buffer.clear();
        }

        void setMemoryID(const MemoryID& id) override
        {
            MemoryBase<_CoreSegmentT>::setMemoryID(id);
            buffer.name() = this->name();
        }

    protected:
        virtual void _directlyStore(const armem::wm::Memory& memory) = 0;
        void _store(const armem::wm::Memory& memory) override
        {
            std::lock_guard l(this->ltm_mutex);
            buffer.append(memory);
        }

    protected:

        /// Internal memory for data consolidated from wm to ltm (buffer)
        /// The to-put-to-ltm buffer (contains data in plain text)
        armem::wm::Memory buffer;

    };
} // namespace armarx::armem::server::ltm
