#pragma once

#include <functional>

// BaseClass
#include "MemoryItem.h"

#include <RobotAPI/libraries/armem/core/MemoryID.h>
#include <RobotAPI/libraries/armem/core/wm/memory_definitions.h>
#include <RobotAPI/libraries/armem/core/wm/aron_conversions.h>

#include "../extractor/Extractor.h"
#include "../converter/dict/Converter.h"
#include "../converter/image/Converter.h"

namespace armarx::armem::server::ltm
{
    /// @brief Interface functions for the longterm memory classes
    class EntitySnapshotBase : public MemoryItem
    {
    public:

        using MemoryItem::MemoryItem;

        /// return the full sub-ltm as a wm::EntitySnapshot with only references
        /// the ltm may be huge, use with caution
        void loadAll(armem::wm::EntitySnapshot& e) const
        {
            _loadAll(e);
        }

        /// convert the references of the input into a wm::Memory
        void load(armem::wm::EntitySnapshot& e) const
        {
            _load(e);
        }

        /// encode the content of a wm::Memory and store
        void store(const armem::wm::EntitySnapshot& e) const
        {
            _store(e);
        }

        static std::string getLevelName();

    protected:
        virtual void _loadAll(armem::wm::EntitySnapshot&) const = 0;
        virtual void _load(armem::wm::EntitySnapshot&) const = 0;
        virtual void _store(const armem::wm::EntitySnapshot&) const = 0;

    protected:
        mutable std::recursive_mutex ltm_mutex;

        DictConverterPtr dictConverter;
        ImageConverterPtr imgConverter;
        ExtractorPtr extractor;
    };
} // namespace armarx::armem::server::ltm
