#pragma once

#include <functional>

// BaseClass
#include "MemoryItem.h"

// ChildType
#include "CoreSegmentBase.h"

// Members
#include "../filter/Filter.h"

// ArmarX
#include <ArmarXCore/core/time/TimeUtil.h>
#include <ArmarXCore/core/logging/LoggingUtil.h>
#include <RobotAPI/libraries/armem/core/MemoryID.h>
#include <RobotAPI/libraries/armem/core/wm/memory_definitions.h>
#include <RobotAPI/libraries/armem/core/wm/aron_conversions.h>

namespace armarx::armem::server::ltm
{
    /// @brief Interface functions for the longterm memory classes
    template <class _CoreSegmentT>
    class MemoryBase : public MemoryItem
    {
    public:
        using CoreSegmentT = _CoreSegmentT;

        using MemoryItem::MemoryItem;

        /// return the full ltm as a wm::Memory with only references
        /// the ltm may be huge, use with caution
        void loadAll(armem::wm::Memory& memory)
        {
            TIMING_START(LTM_Memory_LoadAll);
            _loadAll(memory);
            TIMING_END(LTM_Memory_LoadAll);
        }

        /// convert the references of the input into a wm::Memory
        void load(armem::wm::Memory& memory)
        {
            TIMING_START(LTM_Memory_Load);
            _load(memory);
            TIMING_END(LTM_Memory_Load);
        }

        /// append a wm::Memory instance to the ltm
        void store(const armem::wm::Memory& memory, bool force = false)
        {
            TIMING_START(LTM_Memory_Append);
            if (force or filter->accept(memory))
            {
                _store(memory);
            }
            TIMING_END(LTM_Memory_Append);
        }

        /// iterate over all core segments of this ltm
        virtual bool forEachCoreSegment(std::function<void(CoreSegmentT&)>&& func) const = 0;

        /// find core segment
        virtual std::shared_ptr<CoreSegmentT> findCoreSegment(const std::string&) const = 0;

        /// get level name
        static std::string getLevelName()
        {
            return "LT-Memory";
        }

    protected:
        virtual void _loadAll(armem::wm::Memory& memory) = 0;
        virtual void _load(armem::wm::Memory& memory) = 0;
        virtual void _store(const armem::wm::Memory& memory) = 0;

    protected:
        mutable std::recursive_mutex ltm_mutex;

        FilterPtr filter;
        // ForgetterPtr forgetter;
    };
} // namespace armarx::armem::server::ltm
