#include "ImageExtractor.h"


namespace armarx::armem::server::ltm::extractor
{
    void ImageExtractorVisitor::visitNDArray(const std::string& key, Input& parent, Input& variant)
    {
        ARMARX_CHECK_NOT_NULL(parent);
        ARMARX_CHECK_NOT_NULL(variant);

        auto ndarray = aron::data::NDArray::DynamicCastAndCheck(variant);
        if (ndarray->getShape().size() == 3)
        {
            images[key] = ndarray;
            switch (parent->getDescriptor())
            {
                case aron::data::Descriptor::eDict:
                {
                    auto dict = aron::data::Dict::DynamicCastAndCheck(parent);
                    dict->setElement(key, nullptr);
                    break;
                }
                case aron::data::Descriptor::eList:
                {
                    auto list = aron::data::List::DynamicCastAndCheck(parent);
                    list->setElement(std::stoi(key), nullptr);
                    break;
                }
                default:
                {
                    throw aron::error::AronException(__PRETTY_FUNCTION__, "Found invalid parent in visitor.");
                }
            }
        }
    }

    Extractor::Extraction ImageExtractor::extract(aron::data::DictPtr& data)
    {
        ImageExtractorVisitor visitor;
        aron::data::VariantPtr var = std::static_pointer_cast<aron::data::Variant>(data);
        aron::data::VariantPtr p;
        aron::data::visitRecursive(visitor, var, "", p);

        Extraction encoding;
        encoding.data = data;
        encoding.images = visitor.images;
        return encoding;
    }

    aron::data::DictPtr ImageExtractor::merge(Extraction& encoding)
    {
        return encoding.data;
    }
}
