// Header
#include "CoreSegment.h"

// ArmarX
#include <ArmarXCore/core/time/TimeUtil.h>
#include <ArmarXCore/core/logging/Logging.h>

namespace armarx::armem::server::ltm::disk
{
    namespace
    {
        MemoryID getMemoryIDFromPath(const std::filesystem::path& p)
        {
            util::ensureFolderExists(p);

            MemoryID m;
            m.memoryName = p.parent_path().filename();
            m.coreSegmentName = p.filename();
            return m;
        }
    }

    CoreSegment::CoreSegment(const std::filesystem::path& p) :
        CoreSegmentBase(getMemoryIDFromPath(p)),
        DiskStorage(p)
    {
    }

    bool CoreSegment::forEachProviderSegment(std::function<void(ProviderSegment&)>&& func) const
    {
        if (!checkPath())
        {
            return false;
        }

        std::filesystem::path p = path;
        util::ensureFolderExists(p, false);

        for (const auto& subdir : std::filesystem::directory_iterator(p))
        {
            std::filesystem::path subdirPath = subdir.path();
            ProviderSegment c(subdirPath);
            func(c);
        }
        return true;
    }

    std::shared_ptr<ProviderSegment> CoreSegment::findProviderSegment(const std::string& n) const
    {
        if (!checkPath())
        {
            return {};
        }

        std::filesystem::path p = path;
        util::ensureFolderExists(p, false);

        std::filesystem::path subpath = p / n;
        util::ensureFolderExists(subpath, false);
        auto c = std::make_shared<ProviderSegment>(subpath);
        return c;
    }

    std::string CoreSegment::getExpectedFolderName() const
    {
        return name();
    }

    void CoreSegment::_loadAll(armem::wm::CoreSegment& e)
    {
        e.id() = id();

        forEachProviderSegment([&e](ProviderSegment& x) {
            armem::wm::ProviderSegment s;
            x.loadAll(s);
            e.addProviderSegment(s);
        });
    }

    void CoreSegment::_load(armem::wm::CoreSegment& c)
    {
        c.forEachProviderSegment([this](armem::wm::ProviderSegment& e)
        {
            util::ensureFolderExists(std::filesystem::path(path) / e.id().providerSegmentName, false);
            ProviderSegment c(std::filesystem::path(path) / e.id().providerSegmentName);
            c.load(e);
        });
    }

    void CoreSegment::_store(const armem::wm::CoreSegment& c)
    {
        c.forEachProviderSegment([this](const armem::wm::ProviderSegment& e)
        {
            util::ensureFolderExists(std::filesystem::path(path) / e.id().providerSegmentName);
            ProviderSegment c(std::filesystem::path(path) / e.id().providerSegmentName);
            c.store(e);
        });
    }
}
