// Header
#include "Entity.h"

// ArmarX
#include <ArmarXCore/core/time/TimeUtil.h>
#include <ArmarXCore/core/logging/Logging.h>

namespace armarx::armem::server::ltm::disk
{

    namespace
    {
        MemoryID getMemoryIDFromPath(const std::filesystem::path& p)
        {
            util::ensureFolderExists(p);

            MemoryID m;
            m.memoryName = p.parent_path().parent_path().parent_path().filename();
            m.coreSegmentName = p.parent_path().parent_path().filename();
            m.providerSegmentName = p.parent_path().filename();
            m.entityName = p.filename();
            return m;
        }
    }

    Entity::Entity(const std::filesystem::path& p) :
        EntityBase(getMemoryIDFromPath(p)),
        DiskStorage(p)
    {
    }

    std::string Entity::getExpectedFolderName() const
    {
        return name();
    }

    bool Entity::forEachSnapshot(std::function<void(EntitySnapshot&)>&& func) const
    {
        if (!checkPath())
        {
            return false;
        }

        std::filesystem::path p = path;
        util::ensureFolderExists(p, false);

        for (const auto& subdir : std::filesystem::directory_iterator(p))
        {
            std::filesystem::path subdirPath = subdir.path();
            EntitySnapshot c(subdirPath);
            func(c);
        }
        return true;
    }

    bool Entity::forEachSnapshotInIndexRange(long first, long last, std::function<void(EntitySnapshot&)>&& func) const
    {
        return true;
    }

    bool Entity::forEachSnapshotInTimeRange(const Time& min, const Time& max, std::function<void(EntitySnapshot&)>&& func) const
    {
        return true;
    }

    bool Entity::forEachSnapshotBeforeOrAt(const Time& time, std::function<void(EntitySnapshot&)>&& func) const
    {
        return true;
    }

    bool Entity::forEachSnapshotBefore(const Time& time, std::function<void(EntitySnapshot&)>&& func) const
    {
        return true;
    }

    std::shared_ptr<EntitySnapshot> Entity::findSnapshot(const Time& n) const
    {
        if (!checkPath())
        {
            return {};
        }

        std::filesystem::path p = path;
        util::ensureFolderExists(p, false);

        std::filesystem::path subpath = p / std::to_string(n.toMicroSeconds());
        util::ensureFolderExists(subpath, false);
        auto c = std::make_shared<EntitySnapshot>(subpath);
        return c;
    }

    std::shared_ptr<EntitySnapshot> Entity::findLatestSnapshot() const
    {
        return {};
    }

    std::shared_ptr<EntitySnapshot> Entity::findLatestSnapshotBefore(const Time& time) const
    {
        return {};
    }

    std::shared_ptr<EntitySnapshot> Entity::findLatestSnapshotBeforeOrAt(const Time& time) const
    {
        return {};
    }

    std::shared_ptr<EntitySnapshot> Entity::findFirstSnapshotAfter(const Time& time) const
    {
        return {};
    }

    std::shared_ptr<EntitySnapshot> Entity::findFirstSnapshotAfterOrAt(const Time& time) const
    {
        return {};
    }

    void Entity::_loadAll(armem::wm::Entity& e)
    {
        e.id() = id();

        forEachSnapshot([&e](EntitySnapshotBase& x) {
            armem::wm::EntitySnapshot s;
            x.loadAll(s);
            e.addSnapshot(s);
        });
    }

    void Entity::_load(armem::wm::Entity& p)
    {
        p.forEachSnapshot([this](armem::wm::EntitySnapshot& e)
        {
            util::ensureFolderExists(std::filesystem::path(path) / std::to_string(e.id().timestamp.toMicroSeconds()), false);
            EntitySnapshot c(std::filesystem::path(path) / std::to_string(e.id().timestamp.toMicroSeconds()));
            c.load(e);
        });
    }

    void Entity::_store(const armem::wm::Entity& c)
    {
        c.forEachSnapshot([this](armem::wm::EntitySnapshot& e)
        {
            util::ensureFolderExists(std::filesystem::path(path) / std::to_string(e.id().timestamp.toMicroSeconds()));
            EntitySnapshot c(std::filesystem::path(path) / std::to_string(e.id().timestamp.toMicroSeconds()));
            c.store(e);
        });
    }
}
