// Header
#include "Memory.h"

// ArmarX
#include <ArmarXCore/core/time/TimeUtil.h>
#include <ArmarXCore/core/logging/Logging.h>

namespace armarx::armem::server::ltm::disk
{
    namespace
    {
        MemoryID getMemoryIDFromPath(const std::filesystem::path& p)
        {
            util::ensureFolderExists(p);

            MemoryID m;
            m.memoryName = p.filename();
            return m;
        }
    }

    Memory::Memory() :
        BufferedMemoryBase(),
        DiskStorage()
    {
        filter = std::make_shared<filter::FrequencyFilter>();
        setMemoryID(MemoryID("Test", ""));

        /*std::string armarx_home = std::string(getenv("HOME")) + "/.armarx";
        if (getenv("ARMARX_DEFAULTS_DIR"))
        {
            armarx_home = getenv("ARMARX_DEFAULTS_DIR");
        }
        path = armarx_home + "/armem/disk/data/db";*/
        path = "/tmp/MemoryExport/" + getExpectedFolderName();
    }

    Memory::Memory(const std::filesystem::path& p) :
        BufferedMemoryBase(),
        DiskStorage(p)
    {
        filter = std::make_shared<filter::FrequencyFilter>();
        setMemoryID(getMemoryIDFromPath(p));
    }

    std::string Memory::getExpectedFolderName() const
    {
        return id().memoryName;
    }

    bool Memory::forEachCoreSegment(std::function<void(CoreSegment&)>&& func) const
    {
        if (!checkPath())
        {
            return false;
        }

        std::filesystem::path p = path;
        util::ensureFolderExists(p, false);

        for (const auto& subdir : std::filesystem::directory_iterator(p))
        {
            std::filesystem::path subdirPath = subdir.path();
            CoreSegment c(subdirPath);
            func(c);
        }
        return true;
    }

    std::shared_ptr<CoreSegment> Memory::findCoreSegment(const std::string& n) const
    {
        if (!checkPath())
        {
            return {};
        }

        std::filesystem::path p = path;
        util::ensureFolderExists(p, false);

        std::filesystem::path subpath = p / n;
        util::ensureFolderExists(subpath, false);
        auto c = std::make_shared<CoreSegment>(subpath);
        return c;
    }

    void Memory::_loadAll(armem::wm::Memory& m)
    {
        m.id() = id();

        forEachCoreSegment([&m](CoreSegment& x) {
            armem::wm::CoreSegment s;
            x.loadAll(s);
            m.addCoreSegment(s);
        });
    }

    void Memory::_load(armem::wm::Memory& m)
    {
        if (!checkPath())
        {
            return;
        }

        std::lock_guard l(ltm_mutex);
        m.forEachCoreSegment([this](armem::wm::CoreSegment& e)
        {
            util::ensureFolderExists(std::filesystem::path(path) / e.id().coreSegmentName, false);
            CoreSegment c(std::filesystem::path(path) / e.id().coreSegmentName);
            c.load(e);
        });

    }

    void Memory::_directlyStore(const armem::wm::Memory& m)
    {
        if (!checkPath())
        {
            return;
        }

        std::lock_guard l(ltm_mutex);

        m.forEachCoreSegment([this](const armem::wm::CoreSegment& e)
        {
            util::ensureFolderExists(std::filesystem::path(path) / e.id().coreSegmentName);
            CoreSegment c(std::filesystem::path(path) / e.id().coreSegmentName);
            c.store(e);
        });
    }
}
