// Header
#include "ProviderSegment.h"

// ArmarX
#include <ArmarXCore/core/time/TimeUtil.h>
#include <ArmarXCore/core/logging/Logging.h>

namespace armarx::armem::server::ltm::disk
{
    namespace
    {
        MemoryID getMemoryIDFromPath(const std::filesystem::path& p)
        {
            util::ensureFolderExists(p);

            MemoryID m;
            m.memoryName = p.parent_path().parent_path().filename();
            m.coreSegmentName = p.parent_path().filename();
            m.providerSegmentName = p.filename();
            return m;
        }
    }

    ProviderSegment::ProviderSegment(const std::filesystem::path& p) :
        ProviderSegmentBase(getMemoryIDFromPath(p)),
        DiskStorage(p)
    {
    }

    bool ProviderSegment::forEachEntity(std::function<void(Entity&)>&& func) const
    {
        if (!checkPath())
        {
            return false;
        }

        std::filesystem::path p = path;
        util::ensureFolderExists(p, false);

        for (const auto& subdir : std::filesystem::directory_iterator(p))
        {
            std::filesystem::path subdirPath = subdir.path();
            Entity c(subdirPath);
            func(c);
        }
        return true;
    }

    std::shared_ptr<Entity> ProviderSegment::findEntity(const std::string& n) const
    {
        if (!checkPath())
        {
            return {};
        }

        std::filesystem::path p = path;
        util::ensureFolderExists(p, false);

        std::filesystem::path subpath = p / n;
        util::ensureFolderExists(subpath, false);
        auto c = std::make_shared<Entity>(subpath);
        return c;
    }

    std::string ProviderSegment::getExpectedFolderName() const
    {
        return name();
    }

    void ProviderSegment::_loadAll(armem::wm::ProviderSegment& e)
    {
        e.id() = id();

        forEachEntity([&e](Entity& x) {
            armem::wm::Entity s;
            x.loadAll(s);
            e.addEntity(s);
        });
    }

    void ProviderSegment::_load(armem::wm::ProviderSegment& p)
    {
        p.forEachEntity([this](armem::wm::Entity& e)
        {
            util::ensureFolderExists(std::filesystem::path(path) / e.id().entityName, false);
            Entity c(std::filesystem::path(path) / e.id().entityName);
            c.load(e);
        });
    }

    void ProviderSegment::_store(const armem::wm::ProviderSegment& p)
    {
        p.forEachEntity([this](const armem::wm::Entity& e)
        {
            util::ensureFolderExists(std::filesystem::path(path) / e.id().entityName);
            Entity c(std::filesystem::path(path) / e.id().entityName);
            c.store(e);
        });
    }
}
