// Header
#include "DiskStorage.h"

// ArmarX
#include <ArmarXCore/core/time/TimeUtil.h>
#include <ArmarXCore/core/logging/Logging.h>

namespace armarx::armem::server::ltm::disk
{
    DiskStorage::DiskStorage(const std::filesystem::path& p) :
        path(p.string())
    {
    }

    bool DiskStorage::checkPath() const
    {
        std::filesystem::path memoryP(path);

        // Check connection:
        if (!std::filesystem::exists(path))
        {
            std::filesystem::create_directories(path);
            return true;
        }
        else if (std::string expectedFolderName = getExpectedFolderName(); !std::filesystem::is_directory(path) || memoryP.filename() != expectedFolderName)
        {
            ARMARX_WARNING << deactivateSpam("LTM_PathError_" + expectedFolderName)
                           << "The entered path is not valid. Please use a path leading to a memory folder with name: " << expectedFolderName << "."
                           << "\n\n";
            return false;
        }

        return true;
    }

    void DiskStorage::setPath(const std::string& p)
    {
        path = p;
    }
}
