#include "DataTreeBuilder.h"

#include <QTreeWidgetItem>

#include <RobotAPI/libraries/armem_gui/TreeWidgetBuilder.h>


namespace armarx::armem::gui::instance
{

    DataTreeBuilder::DataTreeBuilder()
    {
    }

    void DataTreeBuilder::updateTree(QTreeWidgetItem* parent, const aron::data::DictPtr& data)
    {
        DictBuilder builder = getDictBuilder();
        builder.setUpdateItemFn([this, &data](const std::string & key, QTreeWidgetItem * item)
        {
            auto child = data->getElement(key);
            this->update(item, key, child);
            return true;
        });

        builder.updateTreeWithContainer(parent, data->getAllKeys());
    }

    void DataTreeBuilder::updateTree(QTreeWidgetItem* parent, const aron::data::ListPtr& data)
    {
        auto children = data->getChildren();

        ListBuilder builder = getListBuilder();
        builder.setUpdateItemFn([this, &children](size_t key, QTreeWidgetItem * item)
        {
            this->update(item, std::to_string(key), children.at(key));
            return true;
        });

        builder.updateTreeWithContainer(parent, getIndex(children.size()));
    }


    void DataTreeBuilder::update(QTreeWidgetItem* item, const std::string& key, const aron::data::VariantPtr& data)
    {
        if (data)
        {
            this->setRowTexts(item, key, data);

            if (auto cast = aron::data::Dict::DynamicCast(data))
            {
                updateTree(item, cast);
            }
            else if (auto cast = aron::data::List::DynamicCast(data))
            {
                updateTree(item, cast);
            }
        }
        else
        {
            // Optional data?
            this->setRowTexts(item, key, "(none)");
        }
    }


}
