#pragma once

#include <sstream>
#include <stack>

#include <QTreeWidget>
#include <QLabel>

#include <RobotAPI/libraries/armem_gui/instance/tree_visitors/TreeDataVisitorBase.h>


namespace armarx::armem::gui::instance
{

    class TreeDataVisitor :
        public aron::data::RecursiveConstVariantVisitor,
        public TreeDataVisitorBase
    {
    public:

        using TreeDataVisitorBase::TreeDataVisitorBase;


        void visitDictOnEnter(const std::string& key, const aron::data::VariantPtr&, const aron::data::VariantPtr& n) override
        {
            _visitEnter(key, "Dict", n->childrenSize());
        }
        void visitDictOnExit(const std::string&, const aron::data::VariantPtr&, const aron::data::VariantPtr&) override
        {
            _visitExit();
        }

        void visitListOnEnter(const std::string& key, const aron::data::VariantPtr&, const aron::data::VariantPtr& n) override
        {
            _visitEnter(key, "List", n->childrenSize());
        }
        void visitListOnExit(const std::string&, const aron::data::VariantPtr&, const aron::data::VariantPtr&) override
        {
            _visitExit();
        }


        void visitBool(const std::string& key, const aron::data::VariantPtr&, const aron::data::VariantPtr& b) override
        {
            const auto x = *aron::data::Bool::DynamicCastAndCheck(b);
            this->addValueRow(key, x, "Bool");
        }
        void visitDouble(const std::string& key, const aron::data::VariantPtr&, const aron::data::VariantPtr& d) override
        {
            const auto x = *aron::data::Double::DynamicCastAndCheck(d);
            this->addValueRow(key, x, "Double");
        }
        void visitFloat(const std::string& key, const aron::data::VariantPtr&, const aron::data::VariantPtr& f) override
        {
            const auto x = *aron::data::Float::DynamicCastAndCheck(f);
            this->addValueRow(key, x, "Float");
        }
        void visitInt(const std::string& key, const aron::data::VariantPtr&, const aron::data::VariantPtr& i) override
        {
            const auto x = *aron::data::Int::DynamicCastAndCheck(i);
            this->addValueRow(key, x, "Int");
        }
        void visitLong(const std::string& key, const aron::data::VariantPtr&, const aron::data::VariantPtr& l) override
        {
            const auto x = *aron::data::Long::DynamicCastAndCheck(l);
            this->addValueRow(key, x, "Long");
        }
        void visitString(const std::string& key, const aron::data::VariantPtr&, const aron::data::VariantPtr& string) override
        {
            const auto x = *aron::data::String::DynamicCastAndCheck(string);
            this->addValueRow(key, x, "String");
        }

        void visitNDArray(const std::string& key, const aron::data::VariantPtr&, const aron::data::VariantPtr& array) override
        {
            const auto x = *aron::data::NDArray::DynamicCastAndCheck(array);
            this->addValueRow(key, x, "ND Array");
        }

    };


}
