#pragma once

#include <map>
#include <vector>
#include <filesystem>

#include <Eigen/Geometry>

#include <IceUtil/Time.h>

#include "ArmarXCore/core/exceptions/local/ExpressionException.h"
#include "RobotAPI/libraries/ArmarXObjects/ObjectID.h"
#include <ArmarXCore/core/PackagePath.h>


namespace armarx::armem::robot
{
    struct RobotDescription
    {
        // IceUtil::Time timestamp;

        std::string name;
        PackagePath xml{"", std::filesystem::path("")};
    };


    struct Twist
    {
        Eigen::Vector3f linear;
        Eigen::Vector3f angular;
    };

    struct PlatformState
    {
        Twist twist;
    };

    struct RobotState
    {
        using JointMap = std::map<std::string, float>;
        using Pose = Eigen::Affine3f;

        IceUtil::Time timestamp;

        Pose globalPose;
        JointMap jointMap;
    };

    struct Robot
    {
        RobotDescription description;
        std::string instance;

        RobotState config;

        IceUtil::Time timestamp;

        std::string name() const 
        {
            ARMARX_CHECK_NOT_EMPTY(description.name) << "The robot name must be set!";
            ARMARX_CHECK_NOT_EMPTY(instance) << "The robot instance name must be provided!";

            return description.name + "/" + instance;
        }
    };

    using Robots = std::vector<Robot>;
    using RobotDescriptions = std::vector<RobotDescription>;
    using RobotStates = std::vector<RobotState>;

    std::ostream& operator<<(std::ostream &os, const RobotDescription &rhs);

}  // namespace armarx::armem::robot
