/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include "../Converter.h"
#include "../../visitor/nlohmannJSON/NlohmannJSONVisitor.h"
#include "../../rw/reader/nlohmannJSON/NlohmannJSONReader.h"

namespace armarx::aron::data
{
    /// Converter struct providing the needed call operators.
    /// WriterImplementation is a writer class, TODO: add concept
    template <class WriterImplementation, class DerivedT>
    struct NlohmannJSONConverter :
            virtual public ConstNlohmannJSONVisitor,
            virtual public Converter<aron::data::reader::NlohmannJSONReader, WriterImplementation, DerivedT>
    {
        virtual ~NlohmannJSONConverter() = default;

        virtual void visitUnknown(const nlohmann::json& o) override
        {
            if (!this->r.readNull(o))
            {
                throw error::AronException(__PRETTY_FUNCTION__, "A visitor got data but the enum is unknown.");
            }
            this->w.writeNull();
        }
    };
}
