/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <memory>
#include <string>

// ArmarX
#include <RobotAPI/interface/aron.h>
#include <RobotAPI/libraries/aron/core/Exception.h>

namespace armarx::aron::data
{
    template <class R>
    class WriterInterface
    {
    public:
        using ReturnType = R;

        virtual ReturnType writeList(const std::vector<ReturnType>& elements) = 0;
        virtual ReturnType writeDict(const std::map<std::string, ReturnType>& elements, const std::optional<ReturnType>& extends = std::nullopt) = 0;

        virtual ReturnType writeNDArray(const std::vector<int>& shape, const std::string& typeAsString, const unsigned char* data) = 0;

        virtual ReturnType writeInt(const int i) = 0;
        virtual ReturnType writeLong(const long i) = 0;
        virtual ReturnType writeFloat(const float i) = 0;
        virtual ReturnType writeDouble(const double i) = 0;
        virtual ReturnType writeString(const std::string& i) = 0;
        virtual ReturnType writeBool(const bool i) = 0;

        virtual ReturnType writeNull() // defaulted implementation
        {
            return {};
        }

        // Convenience methods
        ReturnType writePrimitive(const int i)
        {
            return writeInt(i);
        }

        ReturnType writePrimitive(const long i)
        {
            return writeLong(i);
        }

        ReturnType writePrimitive(const float i)
        {
            return writeFloat(i);
        }

        ReturnType writePrimitive(const double i)
        {
            return writeDouble(i);
        }

        ReturnType writePrimitive(const std::string& i)
        {
            return writeString(i);
        }

        ReturnType writePrimitive(const bool i)
        {
            return writeBool(i);
        }
    };
}
