/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

// STD/STL
#include <memory>
#include <numeric>

// Header
#include "VariantReader.h"

// ArmarX
#include <RobotAPI/libraries/aron/core/Exception.h>
#include <RobotAPI/libraries/aron/core/data/variant/All.h>



namespace armarx::aron::data::reader
{
    void VariantReader::readList(const data::VariantPtr& input, std::vector<data::VariantPtr>& elements)
    {
        auto o = data::List::DynamicCastAndCheck(input);
        elements = o->getElements();
    }

    void VariantReader::readDict(const data::VariantPtr& input, std::map<std::string, data::VariantPtr>& elements)
    {
        auto o = data::Dict::DynamicCastAndCheck(input);
        elements = o->getElements();
    }

    void VariantReader::readNDArray(const data::VariantPtr& input, std::vector<int>& shape, std::string& typeAsString, std::vector<unsigned char>& data)
    {
        auto o = data::NDArray::DynamicCastAndCheck(input);
        shape = o->getShape();
        typeAsString = o->getType();
        data = o->getDataAsVector();
    }

    void VariantReader::readInt(const data::VariantPtr& input, int& i)
    {
        auto o = data::Int::DynamicCastAndCheck(input);
        i = o->getValue();
    }

    void VariantReader::readLong(const data::VariantPtr& input, long& i)
    {
        auto o = data::Long::DynamicCastAndCheck(input);
        i = o->getValue();
    }

    void VariantReader::readFloat(const data::VariantPtr& input, float& i)
    {
        auto o = data::Float::DynamicCastAndCheck(input);
        i = o->getValue();
    }

    void VariantReader::readDouble(const data::VariantPtr& input, double& i)
    {
        auto o = data::Double::DynamicCastAndCheck(input);
        i = o->getValue();
    }

    void VariantReader::readString(const data::VariantPtr& input, std::string& i)
    {
        auto o = data::String::DynamicCastAndCheck(input);
        i = o->getValue();
    }

    void VariantReader::readBool(const data::VariantPtr& input, bool& i)
    {
        auto o = data::Bool::DynamicCastAndCheck(input);
        i = o->getValue();
    }
}
