/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <memory>
#include <stack>

// BaseClass
#include <RobotAPI/libraries/aron/core/data/rw/Reader.h>

// ArmarX
#include <RobotAPI/libraries/aron/core/data/variant/Variant.h>


namespace armarx::aron::data::reader
{
    class VariantReader :
        virtual public ReaderInterface<const data::VariantPtr>
    {
    public:
        // constructors
        VariantReader() = default;

        virtual void readList(const data::VariantPtr& input, std::vector<data::VariantPtr>& elements) override;
        virtual void readDict(const data::VariantPtr& input, std::map<std::string, data::VariantPtr>& elements) override;

        virtual void readNDArray(const data::VariantPtr& input, std::vector<int>& shape, std::string& typeAsString, std::vector<unsigned char>& data) override;

        virtual void readInt(const data::VariantPtr& input, int& i) override;
        virtual void readLong(const data::VariantPtr& input, long& i) override;
        virtual void readFloat(const data::VariantPtr& input, float& i) override;
        virtual void readDouble(const data::VariantPtr& input, double& i) override;
        virtual void readString(const data::VariantPtr& input, std::string& s) override;
        virtual void readBool(const data::VariantPtr& input, bool& i) override;
    };
}
