/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// Header
#include "Double.h"

// ArmarX
#include "Float.h"

namespace armarx::aron::data
{
    /* constructors */
    Double::Double(const data::dto::AronDoublePtr& o, const Path& path) :
        detail::PrimitiveVariant<data::dto::AronDouble, double, Double>::PrimitiveVariant(o, data::Descriptor::eDouble, path)
    {
    }

    Double::Double(const Path& path) :
        detail::PrimitiveVariant<data::dto::AronDouble, double, Double>::PrimitiveVariant(data::Descriptor::eDouble, path)
    {
    }

    Double::Double(const double d, const Path& path) :
        detail::PrimitiveVariant<data::dto::AronDouble, double, Double>::PrimitiveVariant(d, data::Descriptor::eDouble, path)
    {
    }

    /* operators */
    bool Double::operator==(const Double& other) const
    {
        const auto& otherAron = other.toAronDoublePtr();
        if (this->aron->value != otherAron->value)
        {
            return false;
        }
        return true;
    }
    bool Double::operator==(const DoublePtr& other) const
    {
        if (!other)
        {
            return false;
        }
        return *this == *other;
    }

    /* static methods */
    DoublePtr Double::FromAronDoublePtr(const data::dto::AronDoublePtr& aron)
    {
        return std::make_shared<Double>(aron);
    }

    data::dto::AronDoublePtr Double::ToAronDoublePtr(const DoublePtr& navigator)
    {
        return navigator ? navigator->toAronDoublePtr() : nullptr;
    }


    /* public member functions */
    data::dto::AronDoublePtr Double::toAronDoublePtr() const
    {
        return aron;
    }

    /* virtual implementations */
    std::string Double::getShortName() const
    {
        return "Double";
    }
    std::string Double::getFullName() const
    {
        return "armarx::aron::data::Double";
    }

    bool Double::fullfillsType(const type::VariantPtr& type) const
    {
        /*const auto f = FloatNavigator();
        return type->getDescriptor() == type::Descriptor::eDouble || f.fullfillsType(type);*/
        return false;
    }

    type::VariantPtr Double::recalculateType() const
    {
        //return std::make_shared<typenavigator::DoubleNavigator>(getPath());
        return nullptr;
    }
}
