/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::ArmarXObjects::aron
 * @author     Simon Ottenhaus ( simon dot ottenhaus at kit dot edu )
 * @date       2019
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#define BOOST_TEST_MODULE RobotAPI::ArmarXLibraries::aron

#define ARMARX_BOOST_TEST

// STD/STL
#include <iostream>
#include <cstdlib>
#include <ctime>
#include <numeric>

// Test
#include <RobotAPI/Test.h>

// ArmarX
#include <ArmarXCore/libraries/cppgen/CppMethod.h>
#include <ArmarXCore/libraries/cppgen/CppClass.h>
#include <RobotAPI/libraries/aron/core/Exception.h>

// Aron
#include <RobotAPI/libraries/aron/core/test/Randomizer.h>

// Generated File
#include <RobotAPI/libraries/aron/core/test/aron/ListTest.aron.generated.h>
#include <RobotAPI/libraries/aron/core/test/aron/DictTest.aron.generated.h>
#include <RobotAPI/libraries/aron/core/test/aron/PrimitiveTest.aron.generated.h>
#include <RobotAPI/libraries/aron/core/test/aron/ObjectTest.aron.generated.h>
#include <RobotAPI/libraries/aron/core/test/aron/ImageTest.aron.generated.h>
#include <RobotAPI/libraries/aron/core/test/aron/MatrixTest.aron.generated.h>
#include <RobotAPI/libraries/aron/core/test/aron/QuaternionTest.aron.generated.h>
#include <RobotAPI/libraries/aron/core/test/aron/PointCloudTest.aron.generated.h>
#include <RobotAPI/libraries/aron/core/test/aron/PositionTest.aron.generated.h>
#include <RobotAPI/libraries/aron/core/test/aron/OrientationTest.aron.generated.h>
#include <RobotAPI/libraries/aron/core/test/aron/PoseTest.aron.generated.h>
#include <RobotAPI/libraries/aron/core/test/aron/EnumTest.aron.generated.h>
#include <RobotAPI/libraries/aron/core/test/aron/OptionalTest.aron.generated.h>

using namespace armarx;
using namespace aron;

BOOST_AUTO_TEST_CASE(AronCodeGenerationListTest)
{
    std::cout << "Running Code Gen List test" << std::endl;
    ListTest p_tmp;
    ListTest p = p_tmp; // test assignment
    BOOST_CHECK_EQUAL(p_tmp == p, true);

    BOOST_CHECK_EQUAL(typeid(p.intList) == typeid(std::vector<int>), true);
    BOOST_CHECK_EQUAL(typeid(p.longList) == typeid(std::vector<long>), true);
    BOOST_CHECK_EQUAL(typeid(p.floatList) == typeid(std::vector<float>), true);
    BOOST_CHECK_EQUAL(typeid(p.doubleList) == typeid(std::vector<double>), true);
    BOOST_CHECK_EQUAL(typeid(p.stringList) == typeid(std::vector<std::string>), true);
    BOOST_CHECK_EQUAL(typeid(p.boolList) == typeid(std::vector<bool>), true);
    //BOOST_CHECK_EQUAL(typeid(p.objectList) == typeid(std::vector<ListTest::ListClass>), true);

    BOOST_CHECK_EQUAL(p.intList.size() == 0, true);
    BOOST_CHECK_EQUAL(p.longList.size() == 0, true);
    BOOST_CHECK_EQUAL(p.floatList.size() == 0, true);
    BOOST_CHECK_EQUAL(p.doubleList.size() == 0, true);
    BOOST_CHECK_EQUAL(p.stringList.size() == 0, true);
    BOOST_CHECK_EQUAL(p.boolList.size() == 0, true);
    BOOST_CHECK_EQUAL(p.objectList.size() == 0, true);
}

BOOST_AUTO_TEST_CASE(AronCodeGenerationDictTest)
{
    std::cout << "Running Code Gen Dict test" << std::endl;
    DictTest p_tmp;
    DictTest p = p_tmp; // test assignment
    BOOST_CHECK_EQUAL(p_tmp == p, true);

    BOOST_CHECK_EQUAL(p.dict.size() == 0, true);
}

BOOST_AUTO_TEST_CASE(AronCodeGenerationEigenMatrixTest)
{
    std::cout << "Running Code Gen EigenMatrix test" << std::endl;
    EigenMatrixTest p_tmp;
    EigenMatrixTest p = p_tmp; // test assignment
    BOOST_CHECK_EQUAL(p_tmp == p, true);

    BOOST_CHECK_EQUAL(p.the_short_eigen_matrix.rows() == 5, true);
    BOOST_CHECK_EQUAL(p.the_short_eigen_matrix.cols() == 7, true);
    BOOST_CHECK_EQUAL(p.the_int_eigen_matrix.rows() == 7, true);
    BOOST_CHECK_EQUAL(p.the_int_eigen_matrix.cols() == 7, true);
    BOOST_CHECK_EQUAL(p.the_long_eigen_matrix.rows() == 7, true);
    BOOST_CHECK_EQUAL(p.the_long_eigen_matrix.cols() == 5, true);
    BOOST_CHECK_EQUAL(p.the_float_eigen_matrix.rows() == 1, true);
    BOOST_CHECK_EQUAL(p.the_float_eigen_matrix.cols() == 9, true);
    BOOST_CHECK_EQUAL(p.the_double_eigen_matrix.rows() == 25, true);
    BOOST_CHECK_EQUAL(p.the_double_eigen_matrix.cols() == 1, true);
}

BOOST_AUTO_TEST_CASE(AronCodeGenerationEigenQuaternionTest)
{
    std::cout << "Running Code Gen EigenQuaternion test" << std::endl;
    EigenQuaternionTest p_tmp;
    EigenQuaternionTest p = p_tmp; // test assignment
    BOOST_CHECK_EQUAL(p_tmp == p, true);

    //BOOST_CHECK_EQUAL(p.the_double_eigen_matrix.w() == 0, true);
    //BOOST_CHECK_EQUAL(p.the_double_eigen_matrix.x() == 0, true);
    //BOOST_CHECK_EQUAL(p.the_double_eigen_matrix.y() == 0, true);
    //BOOST_CHECK_EQUAL(p.the_double_eigen_matrix.z() == 0, true);
}

BOOST_AUTO_TEST_CASE(AronCodeGenerationEigenPositionTest)
{
    std::cout << "Running Code Gen EigenPosition test" << std::endl;
    PositionTest p_tmp;
    PositionTest p = p_tmp; // test assignment
    BOOST_CHECK_EQUAL(p_tmp == p, true);

    BOOST_CHECK_EQUAL(p.position.rows() == 3, true);
    BOOST_CHECK_EQUAL(p.position.cols() == 1, true);
}

BOOST_AUTO_TEST_CASE(AronCodeGenerationEigenPoseTest)
{
    std::cout << "Running Code Gen EigenPose test" << std::endl;
    PoseTest p_tmp;
    PoseTest p = p_tmp; // test assignment
    BOOST_CHECK_EQUAL(p_tmp == p, true);

    BOOST_CHECK_EQUAL(p.pose.rows() == 4, true);
    BOOST_CHECK_EQUAL(p.pose.cols() == 4, true);
}

BOOST_AUTO_TEST_CASE(AronCodeGenerationIntEnumTest)
{
    std::cout << "Running Code Gen IntEnum test" << std::endl;
    TheObjectThatUsesTheIntEnum p_tmp;
    TheObjectThatUsesTheIntEnum p = p_tmp; // test assignment
    BOOST_CHECK_EQUAL(p_tmp == p, true);

    BOOST_CHECK_EQUAL(p.the_int_enum.value == TheIntEnum::INT_ENUM_VALUE_0, true);
}

BOOST_AUTO_TEST_CASE(AronCodeGenerationPrimitiveTest)
{
    std::cout << "Running Code Gen Primitive test" << std::endl;
    PrimitiveTest p_tmp;
    PrimitiveTest p = p_tmp; // test assignment
    BOOST_CHECK_EQUAL(p_tmp == p, true);

    BOOST_CHECK_EQUAL(p.the_bool == false, true);
    BOOST_CHECK_EQUAL(p.the_double == 0.0, true);
    BOOST_CHECK_EQUAL(p.the_float == 0.0, true);
    BOOST_CHECK_EQUAL(p.the_int == 0, true);
    BOOST_CHECK_EQUAL(p.the_long == 0, true);
    BOOST_CHECK_EQUAL(p.the_string == "", true);
    BOOST_CHECK_EQUAL(p.the_time == IceUtil::Time(), true);
}

BOOST_AUTO_TEST_CASE(AronCodeGenerationOptionalTest)
{
    std::cout << "Running Code Gen Optional test" << std::endl;
    OptionalTest p_tmp;
    OptionalTest p = p_tmp; // test assignment
    BOOST_CHECK_EQUAL(p_tmp == p, true);

    BOOST_CHECK_EQUAL(typeid(p.some_dict) == typeid(std::optional<std::map<std::string, float>>), true);
    BOOST_CHECK_EQUAL(typeid(p.some_dict_with_optional_type) == typeid(std::map<std::string, std::optional<float>>), true);
    BOOST_CHECK_EQUAL(typeid(p.some_eigen_matrix) == typeid(std::optional<Eigen::Matrix<long, 25, 10>>), true);
    BOOST_CHECK_EQUAL(typeid(p.some_float) == typeid(std::optional<float>), true);
    BOOST_CHECK_EQUAL(typeid(p.some_list) == typeid(std::optional<std::vector<double>>), true);
    BOOST_CHECK_EQUAL(typeid(p.some_list_with_optional_list) == typeid(std::optional<std::vector<std::optional<std::vector<std::optional<float>>>>>), true);
    BOOST_CHECK_EQUAL(typeid(p.some_list_with_optional_type) == typeid(std::vector<std::optional<double>>), true);
    BOOST_CHECK_EQUAL(typeid(p.some_obj) == typeid(std::optional<armarx::OptionalTestElement>), true);
    BOOST_CHECK_EQUAL(typeid(p.some_string) == typeid(std::optional<std::string>), true);


    auto aronType = p.toAronType();
    BOOST_CHECK_EQUAL(aronType->getMemberType("some_float")->getMaybe() == aron::type::Maybe::eOptional, true);

    BOOST_CHECK_EQUAL(p.some_float.has_value() == false, true);
    auto aron = p.toAron();
    BOOST_CHECK_EQUAL(aron->getElement("some_float") == nullptr, true);

    p.some_float = 5.0f;
    BOOST_CHECK_EQUAL(p.some_float.has_value() == true, true);
    aron = p.toAron();
    BOOST_CHECK_EQUAL(*aron->getElement("some_float") == std::make_shared<armarx::aron::data::Float>(5.0f), true);
    BOOST_CHECK_EQUAL(*aron->getElement("some_float") == nullptr, false);
}
