/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <memory>
#include <stack>

// Simox
#include <SimoxUtility/json.h>

// BaseClass
#include <RobotAPI/libraries/aron/core/type/rw/Reader.h>

namespace armarx::aron::type::reader
{
    class NlohmannJSONReader :
        public ReaderInterface<nlohmann::json>
    {
    public:
        // constructors
        NlohmannJSONReader() = default;

        void readObject(const nlohmann::json& input, std::string& name, std::map<std::string, nlohmann::json>& memberTypes, type::Maybe& maybe) override;
        void readList(const nlohmann::json& input, nlohmann::json& acceptedType, type::Maybe& maybe) override;
        void readDict(const nlohmann::json& input, nlohmann::json& acceptedType, type::Maybe& maybe) override;
        void readTuple(const nlohmann::json& input, std::vector<nlohmann::json>& acceptedTypes, type::Maybe& maybe) override;
        void readPair(const nlohmann::json& input, std::pair<nlohmann::json, nlohmann::json>& acceptedTypes, type::Maybe& maybe) override;

        void readNDArray(const nlohmann::json& input, int& ndim, type::ndarray::ElementType& type, type::Maybe& maybe) override;
        void readMatrix(const nlohmann::json& input, int& rows, int& cols, type::matrix::ElementType& type, type::Maybe& maybe) override;
        void readQuaternion(const nlohmann::json& input, type::quaternion::ElementType& type, type::Maybe& maybe) override;
        void readPointCloud(const nlohmann::json& input, type::pointcloud::VoxelType& type, type::Maybe& maybe) override;
        void readImage(const nlohmann::json& input, type::image::PixelType& type, type::Maybe& maybe) override;
        void readPosition(const nlohmann::json& input, type::Maybe& maybe) override;
        void readOrientation(const nlohmann::json& input, type::Maybe& maybe) override;
        void readPose(const nlohmann::json& input, type::Maybe& maybe) override;

        void readIntEnum(const nlohmann::json& input, std::string& name, std::map<std::string, int>& acceptedValues, type::Maybe& maybe) override;

        void readInt(const nlohmann::json& input, type::Maybe& maybe) override;
        void readLong(const nlohmann::json& input, type::Maybe& maybe) override;
        void readFloat(const nlohmann::json& input, type::Maybe& maybe) override;
        void readDouble(const nlohmann::json& input, type::Maybe& maybe) override;
        void readString(const nlohmann::json& input, type::Maybe& maybe) override;
        void readBool(const nlohmann::json& input, type::Maybe& maybe) override;
        void readTime(const nlohmann::json& input, type::Maybe& maybe) override;
    };
}
