/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

#include <RobotAPI/libraries/aron/core/type/rw/Writer.h>
#include <RobotAPI/libraries/aron/core/type/variant/Variant.h>

#include <memory>
#include <stack>


namespace armarx::aron::type::writer
{
    class VariantWriter :
        public type::WriterInterface<aron::type::VariantPtr>
    {
    public:
        VariantWriter() = default;

        virtual aron::type::VariantPtr writeObject(const std::string& name, const type::Maybe maybe, const std::map<std::string, aron::type::VariantPtr>& memberTypes, const std::optional<aron::type::VariantPtr>& extends = std::nullopt) override;
        virtual aron::type::VariantPtr writeList(const type::Maybe maybe, const aron::type::VariantPtr& acceptedType) override;
        virtual aron::type::VariantPtr writeDict(const type::Maybe maybe, const aron::type::VariantPtr& acceptedType) override;
        virtual aron::type::VariantPtr writePair(const type::Maybe maybe, const aron::type::VariantPtr& acceptedType1, const aron::type::VariantPtr& acceptedType2) override;
        virtual aron::type::VariantPtr writeTuple(const type::Maybe maybe, const std::vector<aron::type::VariantPtr>& acceptedTypes) override;

        virtual aron::type::VariantPtr writeNDArray(const int ndim, const type::ndarray::ElementType, const type::Maybe maybe) override;
        virtual aron::type::VariantPtr writeMatrix(const int rows, const int cols, const type::matrix::ElementType type, const type::Maybe maybe) override;
        virtual aron::type::VariantPtr writeQuaternion(const type::quaternion::ElementType, const type::Maybe maybe) override;
        virtual aron::type::VariantPtr writeImage(const type::image::PixelType, const type::Maybe maybe) override;
        virtual aron::type::VariantPtr writePointCloud(const type::pointcloud::VoxelType, const type::Maybe maybe) override;
        virtual aron::type::VariantPtr writePosition(const type::Maybe maybe) override;
        virtual aron::type::VariantPtr writeOrientation(const type::Maybe maybe) override;
        virtual aron::type::VariantPtr writePose(const type::Maybe maybe) override;

        virtual aron::type::VariantPtr writeIntEnum(const std::string& name, const std::map<std::string, int>& acceptedValues, const type::Maybe maybe) override;

        virtual aron::type::VariantPtr writeInt(const type::Maybe maybe) override;
        virtual aron::type::VariantPtr writeLong(const type::Maybe maybe) override;
        virtual aron::type::VariantPtr writeFloat(const type::Maybe maybe) override;
        virtual aron::type::VariantPtr writeDouble(const type::Maybe maybe) override;
        virtual aron::type::VariantPtr writeString(const type::Maybe maybe) override;
        virtual aron::type::VariantPtr writeBool(const type::Maybe maybe) override;
        virtual aron::type::VariantPtr writeTime(const type::Maybe maybe) override;
    };
}
